export const ampersand        = 38; // `&`.charCodeAt(0);
export const asterisk         = 42; // `*`.charCodeAt(0);
export const at               = 64; // `@`.charCodeAt(0);
export const comma            = 44; // `,`.charCodeAt(0);
export const colon            = 58; // `:`.charCodeAt(0);
export const semicolon        = 59; // `;`.charCodeAt(0);
export const openParenthesis  = 40; // `(`.charCodeAt(0);
export const closeParenthesis = 41; // `)`.charCodeAt(0);
export const openSquare       = 91; // `[`.charCodeAt(0);
export const closeSquare      = 93; // `]`.charCodeAt(0);
export const dollar           = 36; // `$`.charCodeAt(0);
export const tilde            = 126; // `~`.charCodeAt(0);
export const caret            = 94; // `^`.charCodeAt(0);
export const plus             = 43; // `+`.charCodeAt(0);
export const equals           = 61; // `=`.charCodeAt(0);
export const pipe             = 124; // `|`.charCodeAt(0);
export const greaterThan      = 62; // `>`.charCodeAt(0);
export const space            = 32; // ` `.charCodeAt(0);
export const singleQuote      = 39; // `'`.charCodeAt(0);
export const doubleQuote      = 34; // `"`.charCodeAt(0);
export const slash            = 47; // `/`.charCodeAt(0);
export const bang             = 33; // `!`.charCodeAt(0);

export const backslash        = 92; // '\\'.charCodeAt(0);
export const cr               = 13; // '\r'.charCodeAt(0);
export const feed             = 12; // '\f'.charCodeAt(0);
export const newline          = 10; // '\n'.charCodeAt(0);
export const tab              = 9; // '\t'.charCodeAt(0);

// Expose aliases primarily for readability.
export const str              = singleQuote;

// No good single character representation!
export const comment          = -1;
export const word             = -2;
export const combinator       = -3;
