export const TAG = 'tag';
export const STRING = 'string';
export const SELECTOR = 'selector';
export const ROOT = 'root';
export const PSEUDO = 'pseudo';
export const NESTING = 'nesting';
export const ID = 'id';
export const COMMENT = 'comment';
export const COMBINATOR = 'combinator';
export const CLASS = 'class';
export const ATTRIBUTE = 'attribute';
export const UNIVERSAL = 'universal';
